from abc import ABCMeta, abstractmethod, abstractproperty
import infection_monkey.config
from common.utils.exploit_enum import ExploitType
from datetime import datetime

__author__ = 'itamar'


class HostExploiter(object):
    __metaclass__ = ABCMeta

    _TARGET_OS_TYPE = []

    # Usual values are 'vulnerability' or 'brute_force'
    EXPLOIT_TYPE = ExploitType.VULNERABILITY

    @abstractproperty
    def _EXPLOITED_SERVICE(self):
        pass

    def __init__(self, host):
        self._config = infection_monkey.config.WormConfiguration
        self.exploit_info = {'display_name': self._EXPLOITED_SERVICE,
                              'started': '',
                              'finished': '',
                              'vulnerable_urls': [],
                              'vulnerable_ports': [],
                              'executed_cmds': []}
        self.exploit_attempts = []
        self.host = host

    def set_start_time(self):
        self.exploit_info['started'] = datetime.now().isoformat()

    def set_finish_time(self):
        self.exploit_info['finished'] = datetime.now().isoformat()

    def is_os_supported(self):
        return self.host.os.get('type') in self._TARGET_OS_TYPE

    def send_exploit_telemetry(self, result):
        from infection_monkey.telemetry.exploit_telem import ExploitTelem
        ExploitTelem(self, result).send()

    def report_login_attempt(self, result, user, password='', lm_hash='', ntlm_hash='', ssh_key=''):
        self.exploit_attempts.append({'result': result, 'user': user, 'password': password,
                                       'lm_hash': lm_hash, 'ntlm_hash': ntlm_hash, 'ssh_key': ssh_key})

    def exploit_host(self):
        self.pre_exploit()
        result = self._exploit_host()
        self.post_exploit()
        return result

    def pre_exploit(self):
        self.set_start_time()

    def post_exploit(self):
        self.set_finish_time()

    @abstractmethod
    def _exploit_host(self):
        raise NotImplementedError()

    def add_vuln_url(self, url):
        self.exploit_info['vulnerable_urls'].append(url)

    def add_vuln_port(self, port):
        self.exploit_info['vulnerable_ports'].append(port)

    def add_executed_cmd(self, cmd):
        """
        Appends command to exploiter's info.
        :param cmd: String of executed command. e.g. 'echo Example'
        """
        powershell = True if "powershell" in cmd.lower() else False
        self.exploit_info['executed_cmds'].append({'cmd': cmd, 'powershell': powershell})
        

from infection_monkey.exploit.win_ms08_067 import Ms08_067_Exploiter
from infection_monkey.exploit.wmiexec import WmiExploiter
from infection_monkey.exploit.smbexec import SmbExploiter
from infection_monkey.exploit.sshexec import SSHExploiter
from infection_monkey.exploit.shellshock import ShellShockExploiter
from infection_monkey.exploit.sambacry import SambaCryExploiter
from infection_monkey.exploit.elasticgroovy import ElasticGroovyExploiter
from infection_monkey.exploit.struts2 import Struts2Exploiter
from infection_monkey.exploit.weblogic import WebLogicExploiter
from infection_monkey.exploit.hadoop import HadoopExploiter
from infection_monkey.exploit.mssqlexec import MSSQLExploiter
from infection_monkey.exploit.vsftpd import VSFTPDExploiter
